
*  BVAR_condfor. The code performs the conditional forecast in the paper Canova and Ciccarelli (2007)
*  to reproduce figure 2. (only the BVAR part of figure 2). time-varying coefficients
*  uncertainty is computed with a montecarlo where the posterior of beta is normal with mean equal to the last in sample
*  value of beta obtained with a kalman fitler procedure with time-varying parameters
*
*****************************************************************
seed 11
compute lags=1			;*Number of lags
compute nvar=8 			;*umber of variables
compute ndraws=2500		;*Number of draws
*
cal 1980 1 4
all 14 2002:4
*
open data indexsa.xls
data(org=obs,for=xls) /
close data
/*
* standardization of variables
dofor i = usinf usinfr useg usgdpg jpinf jpinfr jpeg jpgdpg $
  geinf geinfr geeg gegdpg ukinf ukinfr ukeg ukgdpg $
  frinf frinfr freg frgdpg itinf itinfr iteg itgdpg $
  cainf cainfr caeg cagdpg cpg
stat(noprint) ([series] i)
set ([series] i) = (([series] i)-%mean)/sqrt(%variance)
*set ([series] i) = ([series] i)/sqrt(%variance)
end dofor
*
log usinf
log usinfr
log useg
log usgdpg
log jpinf
log jpinfr
log jpeg
log jpgdpg
log geinf
log geinfr
log geeg
log gegdpg
log ukinf
log ukinfr
log ukeg
log ukgdpg
log frinf
log frinfr
log freg
log frgdpg
log itinf
log itinfr
log iteg
log itgdpg
log cainf
log cainfr
log caeg
log cagdpg
*/
** change here for given country
dec index list
enter(varying,entries=nvar) list
# usinf usinfr useg usgdpg geinf geinfr geeg gegdpg

compute beg=1980:1
compute tempend=2000:4
*compute end=2002:4
compute dat=tempend-beg
compute obs=tempend-beg
compute nstep=16			;*Number of response steps
smpl beg tempend

/*
Set up the system
*/
comp tight = .1
compute decay = 10.
compute other = .3
comp tvrel =  0.000001 ;*(5 zeri dopo virgola)
comp contight = 10.
/*
*try other priors use this for italy and canada
comp tight = .2
compute decay = 10.
compute other = .3
comp tvrel =  0.000001
comp contight = 10.
*/
dec vec univar(nvar) univar1(nvar) lagdecay(lags)

*  Compute univariate autoregressions and put SEFACTOR * its regression
*  variance into UNIVAR
*

comp count = 1
comp sefactor = 0.9
do i = 1,nvar
 linreg(noprint) list(i)
  # constant list(i){1 to lags}
  compute univar1(count)=sefactor*%seesq
  comp count = count+1
end do i

declare vector[symm] xxx1 tvx1

compute nvarcos=nvar*lags+1
dim xxx1(nvar) tvx1(nvar)
dec vect cvector(nvarcos)
dec rect cmat(nvarcos,nvar)


system(model=varmodel1) 1 to nvar
variables list
det constant
lags 1 to lags
kfset(noscale,vector=univar1,likelihood=likely) xxx1
tvarying tvx1
end(system)
*  Set the initial coefficient vectors to the random walk mean
*
do i=1,nvar
*  ewise cvector(j)=(j==(i-1)*lags+1)
  ewise cmat(j,i)=(j==(i-1)*lags+1)
end do i
compute %modelsetcoeffs(varmodel1,cmat)

*  Set lag decay. These give relative prior variances, not precisions.
*
ewise lagdecay(i)=1.0
*ewise lagdecay(i)=i**(-2*decayfac)
*ewise lagdecay(i)=decay**(2*(i-1))
*
*  Set up the XXX and TVX matrices

do i=1,nvar
  dim xxx1(i)(nvarcos,nvarcos)
  compute xxx1(i)=%const(0.0)
  do j=1,nvar
    compute basescal=tight**2*%if(i==j,1.0,(univar1(i)*other**2)/univar1(j))
    do l=1,lags
       compute xxx1(i)((j-1)*lags+l,(j-1)*lags+l)=basescal*lagdecay(l)
    end do l
  end do j
  compute xxx1(i)(nvarcos,nvarcos)=contight**2*univar1(i)
  compute xxx1(i)(nvarcos-1,nvarcos-1)=contight**2*univar1(i)
  compute xxx1(i)(nvarcos-2,nvarcos-2)=contight**2*univar1(i)
  compute tvx1(i)=tvrel*xxx1(i)
end do i

do time= beg,tempend
  if time==beg
     kalman(start=time)
  else {
     kalman(coeffs=betaols1,noprint)
     }
 end do time


dec symm vmat1(nvar,nvar)
compute vmat1 = %const(.0)
do i = 1,nvar
compute vmat1(i,i) = univar1(i)
end

*dec symm xxx1_1(nvar*nvarcos,nvar*nvarcos)
comp xxx1_1 = %xx
/*
do co = 1,nvar
do j = 1,nvarcos
do i = 1,nvarcos
comp xxx1_1((co-1)*nvarcos+i,(co-1)*nvarcos+j) = xxx1(co)(i,j)
end
end
end
*/
dec vec[series] fore_inf1(ndraws)

declare rect sxx1 svt1 swish1 betadraw1
declare symm sigmad1

dec vec shock(nvar)
comp shock = %const(.0)
comp shock(3) = shock(4) = 1.0

compute sxx1    =%decomp(xxx1_1)
compute svt1    =%decomp(inv(%nobs*vmat1))
*compute betaols1=%modelgetcoeffs(varmodel1)

compute ncoef1  =%rows(sxx1)
compute wishdof1=%nobs-ncoef1

declare rect ranc1(ncoef1,nvar)
declare vect[rect] responses(ndraws)
declare rect[series] impulses(nvar,nvar)
comp n = 2, g = 4
source(noecho) cndition.src

infobox(action=define,progress,lower=1,upper=ndraws) 'Monte Carlo Integration'
do draws = 1,ndraws
   if %clock(draws,2)==1 {
    compute sigmad1  =%ranwisharti(svt1,wishdof1)
      compute swish1   =%decomp(sigmad1)
      compute ranc1    =%ran(1.0)
      compute betau1   =sxx1*ranc1*tr(swish1)
      compute betadraw1=betaols1+betau1
   }
   else
     compute betadraw1=betaols1-betau1
   compute %modelsetcoeffs(varmodel1,betadraw1)
 @cndition(decomp=swish1) varmodel1 nstep tempend+1 sigmad1 2 condfore
 # usgdpg tempend+1 usgdpg(tempend)+1.
* # usgdpg tempend+2 usgdpg(tempend)+.5
* # usgdpg tempend+3 usgdpg(tempend)+.25
* # usgdpg tempend+4 usgdpg(tempend)
 # useg tempend+1 useg(tempend)+1.
* # useg tempend+2 useg(tempend)+.5
* # useg tempend+3 useg(tempend)+25
* # useg tempend+4 useg(tempend)
 forecast(model=varmodel1,results=forecasts) * nstep tempend+1

/*
set prova tempend+1 tempend+nstep = condfore(4)-forecasts(4)
set prova1 tempend+1 tempend+nstep = condfore(8)-forecasts(8)
graph(key=below) 2 ;#condfore(8) tempend+1 tempend+nstep ;#forecasts(8) tempend+1 tempend+nstep
graph(key=below) 1 ;#prova1 tempend+1 tempend+nstep
*/


*     Store the impulse responses as difference between conditional and unconditional forecast
 dim responses(draws)(nvar,nstep)
 do i = 1,nvar
  do j = 1,nstep
   comp responses(draws)(i,j)=condfore(i)(tempend+j)-forecasts(i)(tempend+j)
  end
 end
/*
   impulse(noprint,model=varmodel1,decomp=swish1,shocks=shock,results=impulses) nvar nstep

*     Store the impulse responses
*   dim responses(draws)(nvar*nvar,nstep)
*   ewise responses(draws)(i,j)=impulses((i-1)/nvar+1,%clock(i,nvar))(j)
   dim responses(draws)(nvar,nstep)
   do i = 1,nvar
    do j = 1,nstep
   comp responses(draws)(i,j)=impulses(i,1)(j)
  end
  end
*/
 infobox(current=draws)
end do draws
infobox(action=remove)

comp i = 4, j = 1

dec vect[series] upper(nvar) lower(nvar) resp(nvar)
   compute minlower=maxupper=0.0
   smpl 1 ndraws
      clear lower(j) upper(j) resp(j)
      do k=1,nstep
*         set work   1 ndraws = responses(t)((i-1)*nvar+j,k)
         set work   1 ndraws = responses(t)(i,k)
         compute frac=%fractiles(work,||.16,0.5,.84||)
         compute lower(j)(k)=frac(1)
         compute upper(j)(k)=frac(3)
         compute resp(j)(k)=frac(2)
      end do k
      compute maxupper=%max(maxupper,%maxvalue(upper(j)))
      compute minlower=%min(minlower,%minvalue(lower(j)))
*
   smpl 1 nstep
      graph(ticks,min=minlower,max=maxupper,number=0) 3 j i
      # resp(j)
      # upper(j) / 2
      # lower(j) / 2
*
print / resp(J) upper(J) lower(J)

comp i = 8, j = 1

dec vect[series] upper(nvar) lower(nvar) resp(nvar)
   compute minlower=maxupper=0.0
   smpl 1 ndraws
      clear lower(j) upper(j) resp(j)
      do k=1,nstep
*         set work   1 ndraws = responses(t)((i-1)*nvar+j,k)
         set work   1 ndraws = responses(t)(i,k)
         compute frac=%fractiles(work,||.16,0.5,.84||)
         compute lower(j)(k)=frac(1)
         compute upper(j)(k)=frac(3)
         compute resp(j)(k)=frac(2)
      end do k
      compute maxupper=%max(maxupper,%maxvalue(upper(j)))
      compute minlower=%min(minlower,%minvalue(lower(j)))
*
   smpl 1 nstep
      graph(ticks,min=minlower,max=maxupper,number=0) 3 j i
      # resp(j)
      # upper(j) / 2
      # lower(j) / 2
print / resp(J) upper(J) lower(J)








