*
*  BVAR_condfor. The code performs the conditional forecast in the paper Canova and Ciccarelli (2007)
*  to reproduce figure 2. (only the BVAR part of figure 2). time-varying coefficients
*  uncertainty is computed with a montecarlo where the posterior of beta is normal with mean equal to the last in sample
*  value of beta obtained with a kalman fitler procedure with time-varying parameters
*
*****************************************************************
seed 11
compute lags=1			;*Number of lags
compute nvar=4			;*Number of variables
compute ndraws=2500		;*Number of draws
*
cal 1980 1 4
all 14 2002:4
*
open data indexsa.xls
data(org=obs,for=xls) /
close data
/*
* standardization of variables
dofor i = usinf usinfr useg usgdpg jpinf jpinfr jpeg jpgdpg $
  geinf geinfr geeg gegdpg ukinf ukinfr ukeg ukgdpg $
  frinf frinfr freg frgdpg itinf itinfr iteg itgdpg $
  cainf cainfr caeg cagdpg cpg
stat(noprint) ([series] i)
set ([series] i) = (([series] i)-%mean)/sqrt(%variance)
*set ([series] i) = ([series] i)/sqrt(%variance)
end dofor
*/
** change here for given country
dec index list
enter(varying,entries=nvar) list
# itinf itinfr iteg itgdpg
*
compute beg=1980:1
compute tempend=1998:4
compute end1=2000:4
compute end=2002:4
compute dat=tempend-beg
compute obs=end-beg
compute nstep=end-tempend			;*Number of response steps
smpl beg end

set cpg1 = cpg
set cpg1 tempend+1 end = 0.5*cpg1{1}
set cpg end1+1 end = 0.5*cpg{1}
set grid 2001:1 2002:4 = (T==2001:1)
*graph(key=below,grid=grid) 2; #cpg tempend 2002:4; #cpg1 tempend 2002:4
/*
Set up the system
*/

comp tight = .2
compute decay = 10.
compute other = 1.
comp tvrel =  0.00000001
comp contight = 10.
/*
comp tight = 0.1
compute decay = 10.
compute other = .3
comp tvrel =  0.000001
comp contight = 10.
*/
dec vec univar(nvar) univar1(nvar) lagdecay(lags)
*  Compute univariate autoregressions and put SEFACTOR * its regression
*  variance into UNIVAR
*

comp count = 1
comp sefactor = 0.9
do i = 1,nvar
 linreg(noprint) list(i) beg tempend
  # constant list(i){1 to lags} cpg1{0 to 1}
  compute univar1(count)=sefactor*%seesq
  comp count = count+1
end do i

declare vector[symm] xxx1 tvx1

compute nvarcos=nvar*lags+3
dim xxx1(nvar) tvx1(nvar)
dec vect cvector(nvarcos)
dec rect cmat(nvarcos,nvar)

system(model=varmodel1) 1 to nvar
variables list
det constant cpg1{0 to 1}
lags 1 to lags
kfset(noscale,vector=univar1,likelihood=likely1) xxx1
tvarying tvx1
end(system)
*  Set the initial coefficient vectors to the random walk mean
*
do i=1,nvar
*  ewise cvector(j)=(j==(i-1)*lags+1)
  ewise cmat(j,i)=(j==(i-1)*lags+1)
end do i
compute %modelsetcoeffs(varmodel1,cmat)

*  Set lag decay. These give relative prior variances, not precisions.
*
ewise lagdecay(i)=1.0
*ewise lagdecay(i)=i**(-2*decayfac)
*ewise lagdecay(i)=decay**(2*(i-1))
*
*  Set up the XXX and TVX matrices
*
do i=1,nvar
  dim xxx1(i)(nvarcos,nvarcos)
  compute xxx1(i)=%const(0.0)
  do j=1,nvar
    compute basescal=tight**2*%if(i==j,1.0,(univar1(i)*other**2)/univar1(j))
    do l=1,lags
       compute xxx1(i)((j-1)*lags+l,(j-1)*lags+l)=basescal*lagdecay(l)
    end do l
  end do j
  compute xxx1(i)(nvarcos,nvarcos)=contight**2*univar1(i)
  compute xxx1(i)(nvarcos-1,nvarcos-1)=contight**2*univar1(i)
  compute xxx1(i)(nvarcos-2,nvarcos-2)=contight**2*univar1(i)
  compute tvx1(i)=tvrel*xxx1(i)
end do i

do time= beg,tempend
  if time==beg
     kalman(start=time)
  else {
     kalman(coeffs=betaols1,noprint)
     }
end do time

dec symm vmat1(nvar,nvar)
compute vmat1 = %const(.0)
do i = 1,nvar
compute vmat1(i,i) = univar1(i)
end

*dec symm xxx1_1(nvar*nvarcos,nvar*nvarcos)
comp xxx1_1 = %xx
/*
do co = 1,nvar
do j = 1,nvarcos
do i = 1,nvarcos
comp xxx1_1((co-1)*nvarcos+i,(co-1)*nvarcos+j) = xxx1(co)(i,j)
end
end
end
*/
dec vec[series] fore_inf1(ndraws)

declare rect sxx1 svt1 swish1 betadraw1
declare symm sigmad1

compute sxx1    =%decomp(xxx1_1)
compute svt1    =%decomp(inv(%nobs*vmat1))
*compute betaols1=%modelgetcoeffs(varmodel1)
*wri coeff betaols1

compute ncoef1  =%rows(sxx1)
compute wishdof1=%nobs-ncoef1

declare rect ranc1(ncoef1,nvar)


**********************

comp count = 1
comp sefactor = 0.9
do i = 1,nvar
 linreg(noprint) list(i) beg tempend
  # constant list(i){1 to lags} cpg{0 to 1}
  compute univar(count)=sefactor*%seesq
  comp count = count+1
end do i

declare vector[symm] xxx tvx

compute nvarcos=nvar*lags+3
dim xxx(nvar) tvx(nvar)
dec vect cvector(nvarcos)
dec rect cmat(nvarcos,nvar)

system(model=varmodel) 1 to nvar
variables list
det constant cpg{0 to 1}
lags 1 to lags
kfset(noscale,vector=univar,likelihood=likely) xxx
tvarying tvx
end(system)
*  Set the initial coefficient vectors to the random walk mean
*
do i=1,nvar
*  ewise cvector(j)=(j==(i-1)*lags+1)
  ewise cmat(j,i)=(j==(i-1)*lags+1)
end do i
compute %modelsetcoeffs(varmodel,cmat)

*  Set lag decay. These give relative prior variances, not precisions.
*
ewise lagdecay(i)=1.0
*ewise lagdecay(i)=i**(-2*decayfac)
*ewise lagdecay(i)=decay**(2*(i-1))
*
*  Set up the XXX and TVX matrices
*
do i=1,nvar
  dim xxx(i)(nvarcos,nvarcos)
  compute xxx(i)=%const(0.0)
  do j=1,nvar
    compute basescal=tight**2*%if(i==j,1.0,(univar(i)*other**2)/univar(j))
    do l=1,lags
       compute xxx(i)((j-1)*lags+l,(j-1)*lags+l)=basescal*lagdecay(l)
    end do l
  end do j
  compute xxx(i)(nvarcos,nvarcos)=contight**2*univar(i)
  compute xxx(i)(nvarcos-1,nvarcos-1)=contight**2*univar(i)
  compute xxx(i)(nvarcos-2,nvarcos-2)=contight**2*univar(i)
  compute tvx(i)=tvrel*xxx(i)
end do i

do time= beg,tempend
  if time==beg
     kalman(start=time)
  else {
     kalman(coeffs=betaols)
     }
end do time

dec symm vmat(nvar,nvar)
compute vmat = %const(.0)
do i = 1,nvar
compute vmat(i,i) = univar(i)
end

*dec symm xxx_1(nvar*nvarcos,nvar*nvarcos)
comp xxx_1 = %xx
/*
do co = 1,nvar
do j = 1,nvarcos
do i = 1,nvarcos
comp xxx_1((co-1)*nvarcos+i,(co-1)*nvarcos+j) = xxx(co)(i,j)
end
end
end
*/
dec vec[series] fore_inf(ndraws)

declare rect sxx svt swish betadraw
declare symm sigmad

compute sxx    =%decomp(xxx_1)
compute svt    =%decomp(inv(%nobs*vmat))
*compute betaols1=%modelgetcoeffs(varmodel1)
*wri coeff betaols1

compute ncoef  =%rows(sxx)
compute wishdof=%nobs-ncoef

declare rect ranc(ncoef,nvar)

infobox(action=define,progress,lower=1,upper=ndraws) 'Monte Carlo Integration'
do draws = 1,ndraws
   if %clock(draws,2)==1 {
      compute sigmad  =%ranwisharti(svt,wishdof)
      compute swish   =%decomp(sigmad)
      compute ranc    =%ran(1.0)
      compute betau   =sxx*ranc*tr(swish)
      compute betadraw=betaols+betau
   }
   else
     compute betadraw=betaols-betau
   compute %modelsetcoeffs(varmodel,betadraw)

 forecast(result=foreca,model=varmodel) nvar nstep tempend+1
 set fore_inf(draws) = foreca(1)
 clear foreca

 infobox(current=draws)
end do draws
infobox(action=remove)

infobox(action=define,progress,lower=1,upper=ndraws) 'Monte Carlo Integration'
do draws = 1,ndraws
   if %clock(draws,2)==1 {
      compute sigmad1  =%ranwisharti(svt1,wishdof1)
      compute swish1   =%decomp(sigmad1)
      compute ranc1    =%ran(1.0)
      compute betau1   =sxx1*ranc1*tr(swish1)
      compute betadraw1=betaols1+betau1
   }
   else
     compute betadraw1=betaols1-betau1
   compute %modelsetcoeffs(varmodel1,betadraw1)

 forecast(result=foreca1,model=varmodel1) nvar nstep tempend+1
 set fore_inf1(draws) = foreca1(1)
 clear foreca1

 infobox(current=draws)
end do draws
infobox(action=remove)

dec vec[series] cond_fore(ndraws)

do draws = 1,ndraws
set cond_fore(draws) = fore_inf(draws)-fore_inf1(draws)
end

dec vec[series] fan_hic(nstep)

do st = 1,nstep
do draws = 1,ndraws
set fan_hic(st) draws draws = cond_fore(draws)(tempend+st)
end
end

smpl
clear fan25_hic
clear fan75_hic fan50_hic
clear fan16_hic
clear fan84_hic
clear fan05_hic
clear fan95_hic
/*
comp fan25_hic(tempend) = fan50_hic(tempend) = fan75_hic(tempend) = ([series] list(1))(tempend)
comp fan05_hic(tempend) = fan95_hic(tempend) = ([series] list(1))(tempend)
comp fan16_hic(tempend) = fan84_hic(tempend) = ([series] list(1))(tempend)
*/
do st = 1,nstep
compute frac=%fractiles(fan_hic(st),||.05,0.95||)
set fan05_hic tempend+st tempend+st = frac(1)
set fan95_hic tempend+st tempend+st  = frac(2)
compute frac=%fractiles(fan_hic(st),||.25,0.5,0.75||)
set fan25_hic tempend+st tempend+st = frac(1)
set fan50_hic tempend+st tempend+st  = frac(2)
set fan75_hic tempend+st tempend+st  = frac(3)
compute frac=%fractiles(fan_hic(st),||.16,0.84||)
set fan16_hic tempend+st tempend+st = frac(1)
set fan84_hic tempend+st tempend+st  = frac(2)
end

smpl tempend+1 end
graph(pattern,header='conditional forecast') 3
# fan50_hic
# fan16_hic
# fan84_hic
/*
graph(pattern,header='conditional forecast') 6
# fan05_hic
# fan95_hic
# fan25_hic
# fan75_hic
# fan16_hic
# fan84_hic
*/
print tempend+1 end fan50_hic fan16_hic fan84_hic



*****************















